process.env.NODE_ENV = 'test'

import chai, { expect } from 'chai'
import { describe, it } from 'mocha'

import ControllerFactory from '../lib'
import { ControllerBase } from '../lib/controller-base'
import { Controller } from '../lib/controller'
const { User } = require('../dev/api/user/model')

const should = chai.should() // eslint-disable-line

describe('ControllerFactory', () => {
  it('it should throw if instantiated by invoking a constructor with `new`', () => {
    const instantiateFactory = () => new ControllerFactory()
    const errorMessage = 'ControllerFactory includes only static methods and does not need to be instantiated' // eslint-disable-line

    expect(instantiateFactory).to.throw(Error, errorMessage)
  })

  describe('ControllerFactory.getControllerFor(User)', () => {
    it('it should return a Controller instance with base CRUD methods for User schema', () => {
      const controller = ControllerFactory.getControllerFor(User)

      controller.should.be.an('object')
      expect(controller).to.be.an.instanceof(Controller)

      testControllerModel(controller)
      testControllerSignature(controller)
    })

    it('it should return a Controller instance with overridden create method for User schema', () => { // eslint-disable-line
      const override = { create: 'Overridden prop' }
      const controller = ControllerFactory.getControllerFor(User, override)

      controller.should.be.an('object')
      expect(controller).to.be.an.instanceof(Controller)

      testControllerModel(controller)
      testControllerSignature(controller)

      controller.create.should.be.eql(override.create)
    })
  })

  describe('ControllerFactory.setControllerInterface()', () => {
    it('it should throw if first argument is not an instance of Controller', () => { // eslint-disable-line
      const initial = { method: function initialMethod() {} }
      const primaryMixin = { prop: 'property' }
      const secondaryMixin = { method: function method() {} }

      const setInterface = () => ControllerFactory
        .setControllerInterface(initial, secondaryMixin, primaryMixin)

      expect(setInterface).to.throw(Error, 'Object provided is not a Controller instance')
    })
  })

  describe('ControllerFactory.setControllerInterface()', () => {
    it('it should merge a Controller instance`s prototype with given mixins', () => { // eslint-disable-line
      const expected = new Controller(User)
      const expectedProto = Object.getPrototypeOf(expected)
      expectedProto.method = function method() {}
      expectedProto.prop = 'property'

      const actual = new Controller(User)
      const primaryMixin = { prop: 'property' }
      const secondaryMixin = { method: function method() {} }

      ControllerFactory.setControllerInterface(actual, secondaryMixin, primaryMixin)

      actual.should.be.an('object')
      expect(actual).to.be.an.instanceof(Controller)
      actual.should.have.property('method')
      actual.method.constructor.name.should.be.eql(expected.method.constructor.name)
      actual.should.have.property('prop')
      actual.prop.constructor.name.should.be.eql(expected.prop.constructor.name)
    })
  })
})

function testControllerSignature(controller) {
  const controllerProto = Object.getPrototypeOf(controller)
  const expected = Object.keys(ControllerBase)
  const actual = Object.keys(controllerProto)

  expected.forEach(prop => {
    expect(actual).to.contain(prop)
  })
}

function testControllerModel(controller) {
  const expected = User.toString()
  const actual = controller.model.toString()

  expected.should.be.eql(actual)
}
