process.env.NODE_ENV = 'test'

import chai from 'chai'
import chaiHttp from 'chai-http'
import { describe, beforeEach, it } from 'mocha'

const { server } = require('../dev/server')
const { User } = require('../dev/api/user/model')

const should = chai.should() // eslint-disable-line
chai.use(chaiHttp)

describe('Controller for User schema', () => {
  beforeEach((done) => {
    User.remove({}, () => {
      done()
    })
  })

  describe('/GET users', () => {
    it('it should GET all the users with metadata', (done) => {
      chai.request(server)
        .get('/users')
        .end((error, response) => {
          response.should.have.status(200)

          response.body.should.be.an('object')

          const { users, metadata } = response.body

          users.should.be.an('array')
          users.length.should.be.eql(0)

          metadata.should.be.an('object')
          metadata.total.should.be.a('number')
          metadata.total.should.equal(0)

          done()
        })
    })
  })

  describe('/POST users', () => {
    it('it should not POST a user without a name', (done) => {
      chai.request(server)
        .post('/users')
        .send({})
        .end((error, response) => {
          response.should.have.status(400)
          response.body.should.be.an('object')
          response.body.error.should.have.property('errors')
          response.body.error.errors.should.have.property('name')
          response.body.error.errors.name.should.have.property('kind').eql('required')
          done()
        })
    })

    it('it should POST a user', (done) => {
      const mockUser = { name: 'John Doe' }

      chai.request(server)
        .post('/users')
        .send(mockUser)
        .end((error, response) => {
          response.should.have.status(201)

          testUser(response.body, mockUser)

          done()
        })
    })
  })

  describe('/GET/:id users', () => {
    it('it should GET a user by the given id', (done) => {
      const mockUser = new User({ name: 'Alexei Eerik' })

      mockUser.save((error, user) => {
        chai.request(server)
          .get(`/users/${user._id}`)
          .send(user)
          .end((findError, response) => {
            response.should.have.status(200)

            testUser(response.body, mockUser)

            done()
          })
      })
    })
  })

  describe('/PUT/:id users', () => {
    it('it should UPDATE a user by given id', (done) => {
      const mockUser = new User({ name: 'Roydon Nethanel' })
      const updateBody = { name: 'Roydon Andreas' }

      mockUser.save((error, user) => {
        chai.request(server)
          .put(`/users/${user._id}`)
          .send(updateBody)
          .end((updateError, response) => {
            response.should.have.status(200)

            testUser(response.body, updateBody)

            done()
          })
      })
    })
  })

  describe('/DELETE/:id users', () => {
    it('it should DELETE a user by given id', (done) => {
      const mockUser = new User({ name: 'Kai Karsten' })

      mockUser.save((error, user) => {
        chai.request(server)
          .delete(`/users/${user._id}`)
          .end((deleteError, response) => {
            response.should.have.status(200)

            response.body.should.be.a('object')
            response.body.should.have.property('user').equal(user._id.toString())

            done()
          })
      })
    })
  })
})

function testUser(actualUser, expectedUser) {
  actualUser.should.be.an('object')
  actualUser.should.have.property('name')
  actualUser.name.should.be.eql(expectedUser.name)
}
